@@ -151,7 +151,8 @@ _compute_transform (cairo_win32_scaled_font_t *scaled_font,
 	    scaled_font->y_scale = - scaled_font->y_scale;
 
 	scaled_font->logical_scale = WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale;
-	scaled_font->logical_size = WIN32_FONT_LOGICAL_SCALE * floor (scaled_font->y_scale + 0.5);
+	scaled_font->logical_size = WIN32_FONT_LOGICAL_SCALE *
+                                    _cairo_lround (scaled_font->y_scale);
     }
 
     /* The font matrix has x and y "scale" components which we extract and
@@ -165,7 +166,8 @@ _compute_transform (cairo_win32_scaled_font_t *scaled_font,
 					     &scaled_font->x_scale, &scaled_font->y_scale,
 					     TRUE);	/* XXX: Handle vertical text */
 
-	scaled_font->logical_size = floor (WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale + 0.5);
+	scaled_font->logical_size = _cairo_lround (WIN32_FONT_LOGICAL_SCALE *
+                                                   scaled_font->y_scale);
 	scaled_font->logical_scale = WIN32_FONT_LOGICAL_SCALE * scaled_font->y_scale;
     }
 
@@ -237,6 +239,8 @@ _win32_scaled_font_create (LOGFONTW                   *logfont,
     cairo_matrix_t scale;
     cairo_status_t status;
 
+    _cairo_win32_initialize ();
+
     f = malloc (sizeof(cairo_win32_scaled_font_t));
     if (f == NULL)
 	return NULL;
@@ -468,6 +472,8 @@ _cairo_win32_scaled_font_create_toy (cairo_toy_font_face_t *toy_face,
     int face_name_len;
     cairo_status_t status;
 
+    _cairo_win32_initialize ();
+
     status = _cairo_utf8_to_utf16 (toy_face->family, -1,
 				   &face_name, &face_name_len);
     if (status)
@@ -687,7 +693,7 @@ _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font)
     if (!hdc)
 	return CAIRO_STATUS_NO_MEMORY;
 
-    if (scaled_font->preserve_axes) {
+    if (scaled_font->preserve_axes || scaled_font->base.options.hint_metrics == CAIRO_HINT_METRICS_OFF) {
 	/* For 90-degree rotations (including 0), we get the metrics
 	 * from the GDI in logical space, then convert back to font space
 	 */
@@ -717,7 +723,7 @@ _cairo_win32_scaled_font_set_metrics (cairo_win32_scaled_font_t *scaled_font)
 	_cairo_win32_scaled_font_done_unscaled_font (&scaled_font->base);
 
 	extents.ascent = (double)metrics.tmAscent / scaled_font->em_square;
-	extents.descent = metrics.tmDescent * scaled_font->em_square;
+	extents.descent = (double)metrics.tmDescent / scaled_font->em_square;
 	extents.height = (double)(metrics.tmHeight + metrics.tmExternalLeading) / scaled_font->em_square;
 	extents.max_x_advance = (double)(metrics.tmMaxCharWidth) / scaled_font->em_square;
 	extents.max_y_advance = 0;
@@ -755,7 +761,7 @@ _cairo_win32_scaled_font_init_glyph_metrics (cairo_win32_scaled_font_t *scaled_f
     else
         glyph_index_option = 0;
 
-    if (scaled_font->preserve_axes) {
+    if (scaled_font->preserve_axes && scaled_font->base.options.hint_style != CAIRO_HINT_METRICS_OFF) {
 	/* If we aren't rotating / skewing the axes, then we get the metrics
 	 * from the GDI in device space and convert to font space.
 	 */
@@ -860,8 +866,8 @@ _cairo_win32_scaled_font_glyph_bbox (void		 *abstract_font,
             glyph_index_option = 0;
 
 	for (i = 0; i < num_glyphs; i++) {
-	    int x = floor (0.5 + glyphs[i].x);
-	    int y = floor (0.5 + glyphs[i].y);
+	    int x = _cairo_lround (glyphs[i].x);
+	    int y = _cairo_lround (glyphs[i].y);
 
 	    GetGlyphOutlineW (hdc, glyphs[i].index, GGO_METRICS | glyph_index_option,
 			     &metrics, 0, NULL, &matrix);
@@ -962,8 +968,8 @@ _add_glyph (cairo_glyph_state_t *state,
 
     cairo_matrix_transform_point (&state->scaled_font->device_to_logical, &user_x, &user_y);
 
-    logical_x = floor (user_x + 0.5);
-    logical_y = floor (user_y + 0.5);
+    logical_x = _cairo_lround (user_x);
+    logical_y = _cairo_lround (user_y);
 
     if (state->glyphs.num_elements > 0) {
 	int dx;
@@ -1107,7 +1113,7 @@ _compute_a8_mask (cairo_win32_surface_t *mask_surface)
     return &image8->base;
 }
 
-static cairo_status_t
+static cairo_int_status_t
 _cairo_win32_scaled_font_glyph_init (void		       *abstract_font,
 				     cairo_scaled_glyph_t      *scaled_glyph,
 				     cairo_scaled_glyph_info_t  info)
@@ -1145,7 +1151,7 @@ _cairo_win32_scaled_font_show_glyphs (void		       *abstract_font,
 				      int			dest_y,
 				      unsigned int		width,
 				      unsigned int		height,
-				      const cairo_glyph_t      *glyphs,
+				      cairo_glyph_t	       *glyphs,
 				      int                 	num_glyphs)
 {
     cairo_win32_scaled_font_t *scaled_font = abstract_font;
@@ -1275,10 +1281,32 @@ _cairo_win32_scaled_font_load_truetype_table (void	       *abstract_font,
     return status;
 }
 
-static cairo_fixed_t
-_cairo_fixed_from_FIXED (FIXED f)
+static void
+_cairo_win32_scaled_font_map_glyphs_to_unicode (void *abstract_font,
+                                                      cairo_scaled_font_subset_t *font_subset)
 {
-    return *((cairo_fixed_t *)&f);
+    cairo_win32_scaled_font_t *scaled_font = abstract_font;
+    unsigned int i;
+
+    if (scaled_font->glyph_indexing)
+        return;
+
+    for (i = 0; i < font_subset->num_glyphs; i++)
+        font_subset->to_unicode[i] = font_subset->glyphs[i];
+}
+
+static void
+_cairo_win32_transform_FIXED_to_fixed (cairo_matrix_t *matrix,
+                                       FIXED Fx, FIXED Fy,
+                                       cairo_fixed_t *fx, cairo_fixed_t *fy)
+{
+    double x, y;
+
+    x = _cairo_fixed_to_double (*((cairo_fixed_t *)&Fx));
+    y = _cairo_fixed_to_double (*((cairo_fixed_t *)&Fy));
+    cairo_matrix_transform_point (matrix, &x, &y);
+    *fx =  _cairo_fixed_from_double (x);
+    *fy =  _cairo_fixed_from_double (y);
 }
 
 static cairo_status_t
@@ -1292,6 +1320,8 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
     DWORD bytesGlyph;
     unsigned char *buffer, *ptr;
     cairo_path_fixed_t *path;
+    cairo_matrix_t transform;
+    cairo_fixed_t x, y;
     UINT glyph_index_option;
 
     hdc = _get_global_font_dc ();
@@ -1302,7 +1332,14 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
     if (!path)
 	return CAIRO_STATUS_NO_MEMORY;
 
-    status = cairo_win32_scaled_font_select_font (&scaled_font->base, hdc);
+    if (scaled_font->base.options.hint_style == CAIRO_HINT_STYLE_NONE) {
+        status = _cairo_win32_scaled_font_select_unscaled_font (&scaled_font->base, hdc);
+        transform = scaled_font->base.scale;
+        cairo_matrix_scale (&transform, 1.0/scaled_font->em_square, 1.0/scaled_font->em_square);
+    } else {
+        status = cairo_win32_scaled_font_select_font (&scaled_font->base, hdc);
+        cairo_matrix_init_identity(&transform);
+    }
     if (status)
         goto CLEANUP_PATH;
 
@@ -1341,9 +1378,11 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 
 	ptr += sizeof (TTPOLYGONHEADER);
 
-	_cairo_path_fixed_move_to (path,
-				   _cairo_fixed_from_FIXED (header->pfxStart.x),
-				   _cairo_fixed_from_FIXED (header->pfxStart.y));
+        _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                               header->pfxStart.x,
+                                               header->pfxStart.y,
+                                               &x, &y);
+        _cairo_path_fixed_move_to (path, x, y);
 
 	while (ptr < endPoly) {
 	    TTPOLYCURVE *curve = (TTPOLYCURVE *)ptr;
@@ -1352,26 +1391,36 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 	    switch (curve->wType) {
 	    case TT_PRIM_LINE:
 		for (i = 0; i < curve->cpfx; i++) {
-		    _cairo_path_fixed_line_to (path,
-					       _cairo_fixed_from_FIXED (points[i].x),
-					       _cairo_fixed_from_FIXED (points[i].y));
+                    _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                           points[i].x,
+                                                           points[i].y,
+                                                           &x, &y);
+		    _cairo_path_fixed_line_to (path, x, y);
 		}
 		break;
 	    case TT_PRIM_QSPLINE:
 		for (i = 0; i < curve->cpfx - 1; i++) {
 		    cairo_fixed_t p1x, p1y, p2x, p2y, cx, cy, c1x, c1y, c2x, c2y;
 		    _cairo_path_fixed_get_current_point (path, &p1x, &p1y);
-		    cx = _cairo_fixed_from_FIXED (points[i].x);
-		    cy = _cairo_fixed_from_FIXED (points[i].y);
+                    _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                           points[i].x,
+                                                           points[i].y,
+                                                           &cx, &cy);
 
 		    if (i + 1 == curve->cpfx - 1) {
-			p2x = _cairo_fixed_from_FIXED (points[i + 1].x);
-			p2y = _cairo_fixed_from_FIXED (points[i + 1].y);
+                        _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                               points[i + 1].x,
+                                                               points[i + 1].y,
+                                                               &p2x, &p2y);
 		    } else {
 			/* records with more than one curve use interpolation for
 			   control points, per http://support.microsoft.com/kb/q87115/ */
-			p2x = (cx + _cairo_fixed_from_FIXED (points[i + 1].x)) / 2;
-			p2y = (cy + _cairo_fixed_from_FIXED (points[i + 1].y)) / 2;
+                        _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                               points[i + 1].x,
+                                                               points[i + 1].y,
+                                                               &x, &y);
+                        p2x = (cx + x) / 2;
+			p2y = (cy + y) / 2;
 		    }
 
 		    c1x = 2 * cx / 3 + p1x / 3;
@@ -1384,13 +1433,20 @@ _cairo_win32_scaled_font_init_glyph_path (cairo_win32_scaled_font_t *scaled_font
 		break;
 	    case TT_PRIM_CSPLINE:
 		for (i = 0; i < curve->cpfx - 2; i += 2) {
-		    _cairo_path_fixed_curve_to (path,
-						_cairo_fixed_from_FIXED (points[i].x),
-						_cairo_fixed_from_FIXED (points[i].y),
-						_cairo_fixed_from_FIXED (points[i + 1].x),
-						_cairo_fixed_from_FIXED (points[i + 1].y),
-						_cairo_fixed_from_FIXED (points[i + 2].x),
-						_cairo_fixed_from_FIXED (points[i + 2].y));
+		    cairo_fixed_t x1, y1, x2, y2;
+                    _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                           points[i].x,
+                                                           points[i].y,
+                                                           &x, &y);
+                    _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                           points[i + 1].x,
+                                                           points[i + 1].y,
+                                                           &x1, &y1);
+                    _cairo_win32_transform_FIXED_to_fixed (&transform,
+                                                           points[i + 2].x,
+                                                           points[i + 2].y,
+                                                           &x2, &y2);
+		    _cairo_path_fixed_curve_to (path, x, y, x1, y1, x2, y2);
 		}
 		break;
 	    }
@@ -1425,6 +1481,7 @@ const cairo_scaled_font_backend_t cairo_win32_scaled_font_backend = {
     NULL,			/* ucs4_to_index */
     _cairo_win32_scaled_font_show_glyphs,
     _cairo_win32_scaled_font_load_truetype_table,
+    _cairo_win32_scaled_font_map_glyphs_to_unicode,
 };
 
 /* cairo_win32_font_face_t */
@@ -1453,6 +1510,8 @@ _cairo_win32_font_face_scaled_font_create (void			*abstract_face,
 {
     cairo_win32_font_face_t *font_face = abstract_face;
 
+    _cairo_win32_initialize ();
+
     *font = _win32_scaled_font_create (&font_face->logfont,
 				       font_face->hfont,
 				       &font_face->base,
@@ -1490,6 +1549,8 @@ cairo_win32_font_face_create_for_logfontw (LOGFONTW *logfont)
 {
     cairo_win32_font_face_t *font_face;
 
+    _cairo_win32_initialize ();
+
     font_face = malloc (sizeof (cairo_win32_font_face_t));
     if (!font_face) {
 	_cairo_error (CAIRO_STATUS_NO_MEMORY);
@@ -1523,6 +1584,8 @@ cairo_win32_font_face_create_for_hfont (HFONT font)
 {
     cairo_win32_font_face_t *font_face;
 
+    _cairo_win32_initialize ();
+
     font_face = malloc (sizeof (cairo_win32_font_face_t));
     if (!font_face) {
 	_cairo_error (CAIRO_STATUS_NO_MEMORY);
@@ -1601,7 +1664,7 @@ cairo_win32_scaled_font_select_font (cairo_scaled_font_t *scaled_font,
 
 /**
  * cairo_win32_scaled_font_done_font:
- * @scaled_font: A #cairo_scaled_font_t from the Win32 font backend.
+ * @scaled_font: A scaled font from the Win32 font backend.
  *
  * Releases any resources allocated by cairo_win32_scaled_font_select_font()
  **/
@@ -1612,7 +1675,7 @@ cairo_win32_scaled_font_done_font (cairo_scaled_font_t *scaled_font)
 
 /**
  * cairo_win32_scaled_font_get_metrics_factor:
- * @scaled_font: a #cairo_scaled_font_t from the Win32 font backend
+ * @scaled_font: a scaled font from the Win32 font backend
  *
  * Gets a scale factor between logical coordinates in the coordinate
  * space used by cairo_win32_scaled_font_select_font() (that is, the
@@ -1627,3 +1690,39 @@ cairo_win32_scaled_font_get_metrics_factor (cairo_scaled_font_t *scaled_font)
 {
     return 1. / ((cairo_win32_scaled_font_t *)scaled_font)->logical_scale;
 }
+
+/**
+ * cairo_win32_scaled_font_get_logical_to_device:
+ * @scaled_font: a scaled font from the Win32 font backend
+ * @logical_to_device: matrix to return
+ *
+ * Gets the transformation mapping the logical space used by @scaled_font
+ * to device space.
+ *
+ * Since: 1.4
+ **/
+void
+cairo_win32_scaled_font_get_logical_to_device (cairo_scaled_font_t *scaled_font,
+					       cairo_matrix_t *logical_to_device)
+{
+    cairo_win32_scaled_font_t *win_font = (cairo_win32_scaled_font_t *)scaled_font;
+    *logical_to_device = win_font->logical_to_device;
+}
+
+/**
+ * cairo_win32_scaled_font_get_device_to_logical:
+ * @scaled_font: a scaled font from the Win32 font backend
+ * @device_to_logical: matrix to return
+ *
+ * Gets the transformation mapping device space to the logical space
+ * used by @scaled_font.
+ *
+ * Since: 1.4
+ **/
+void
+cairo_win32_scaled_font_get_device_to_logical (cairo_scaled_font_t *scaled_font,
+					       cairo_matrix_t *device_to_logical)
+{
+    cairo_win32_scaled_font_t *win_font = (cairo_win32_scaled_font_t *)scaled_font;
+    *device_to_logical = win_font->device_to_logical;
+}
