@@ -26,13 +26,8 @@
 #ifndef _CAIRO_TEST_H_
 #define _CAIRO_TEST_H_
 
-#if HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdio.h>
-#include <math.h>
-#include <cairo.h>
+#define CAIRO_BOILERPLATE_LOG(...) cairo_test_log (__VA_ARGS__)
+#include "cairo-boilerplate.h"
 
 CAIRO_BEGIN_DECLS
 
@@ -58,13 +53,12 @@ typedef unsigned __int64 uint64_t;
 #error Cannot find definitions for fixed-width integral types (uint8_t, uint32_t, \etc.)
 #endif
 
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
-#define CAIRO_PRINTF_FORMAT(fmt_index, va_index) \
-	__attribute__((__format__(__printf__, fmt_index, va_index)))
-#else
-#define CAIRO_PRINTF_FORMAT(fmt_index, va_index)
+#ifdef _MSC_VER
+#define _USE_MATH_DEFINES
 #endif
 
+#include <math.h>
+
 typedef enum cairo_test_status {
     CAIRO_TEST_SUCCESS = 0,
     CAIRO_TEST_FAILURE,
@@ -107,17 +101,21 @@ typedef struct _cairo_test {
 cairo_test_status_t
 cairo_test (cairo_test_t *test);
 
-/* cairo_test_init() and cairo_test_log() exist to help in writing
- * tests for which cairo_test() is not appropriate for one reason or
- * another. For example, some tests might not be doing any drawing at
- * all, or may need to create their own cairo_t rather than be handed
- * one by cairo_test.
+/* cairo_test_init(), cairo_test_log(), and cairo_test_fini() exist to
+ * help in writing tests for which cairo_test() is not appropriate for
+ * one reason or another. For example, some tests might not be doing
+ * any drawing at all, or may need to create their own cairo_t rather
+ * than be handed one by cairo_test.
  */
 
 /* Initialize test-specific resources, (log files, etc.) */
 void
 cairo_test_init (const char *test_name);
 
+/* Finalize test-specific resource. */
+void
+cairo_test_fini (void);
+
 /* Print a message to the log file, ala printf. */
 void
 cairo_test_log (const char *fmt, ...) CAIRO_PRINTF_FORMAT(1, 2);
@@ -135,9 +133,6 @@ cairo_test_create_pattern_from_png (const char *filename);
 cairo_status_t
 cairo_test_paint_checkered (cairo_t *cr);
 
-void
-xasprintf (char **strp, const char *fmt, ...) CAIRO_PRINTF_FORMAT(2, 3);
-
 CAIRO_END_DECLS
 
 #endif
